@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1991-2002 Jamie Zawinski <jwz@netscape.com>
+/* xscreensaver, Copyright (c) 1991-2006 Jamie Zawinski <jwz@netscape.com>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -87,10 +87,7 @@ draw_shaded_rectangle (Display *dpy, Window window,
 int
 string_width (XFontStruct *font, char *s)
 {
-  int direction, ascent, descent;
-  XCharStruct overall;
-  XTextExtents (font, s, strlen(s), &direction, &ascent, &descent, &overall);
-  return overall.width;
+  return XTextWidth(font, s, strlen(s));
 }
 
 
@@ -99,11 +96,11 @@ static void draw_splash_window (saver_info *si);
 static void destroy_splash_window (saver_info *si);
 static void unsplash_timer (XtPointer closure, XtIntervalId *id);
 
-static void do_demo (saver_info *si);
+static void do_demo (saver_screen_info *ssi);
 #ifdef PREFS_BUTTON
-static void do_prefs (saver_info *si);
+static void do_prefs (saver_screen_info *ssi);
 #endif /* PREFS_BUTTON */
-static void do_help (saver_info *si);
+static void do_help (saver_screen_info *ssi);
 
 
 struct splash_dialog_data {
@@ -147,6 +144,7 @@ struct splash_dialog_data {
   Dimension help_button_x, help_button_y;
 
   Pixmap logo_pixmap;
+  Pixmap logo_clipmask;
   int logo_npixels;
   unsigned long *logo_pixels;
 
@@ -177,19 +175,25 @@ make_splash_dialog (saver_info *si)
   sp = (splash_dialog_data *) calloc (1, sizeof(*sp));
   sp->prompt_screen = ssi;
 
-  sp->heading_label = get_string_resource ("splash.heading.label",
+  sp->heading_label = get_string_resource (si->dpy,
+                                           "splash.heading.label",
 					   "Dialog.Label.Label");
-  sp->body_label = get_string_resource ("splash.body.label",
+  sp->body_label = get_string_resource (si->dpy,
+                                        "splash.body.label",
 					"Dialog.Label.Label");
-  sp->body2_label = get_string_resource ("splash.body2.label",
+  sp->body2_label = get_string_resource (si->dpy,
+                                         "splash.body2.label",
 					 "Dialog.Label.Label");
-  sp->demo_label = get_string_resource ("splash.demo.label",
+  sp->demo_label = get_string_resource (si->dpy,
+                                        "splash.demo.label",
 					"Dialog.Button.Label");
 #ifdef PREFS_BUTTON
-  sp->prefs_label = get_string_resource ("splash.prefs.label",
+  sp->prefs_label = get_string_resource (si->dpy,
+                                         "splash.prefs.label",
 					"Dialog.Button.Label");
 #endif /* PREFS_BUTTON */
-  sp->help_label = get_string_resource ("splash.help.label",
+  sp->help_label = get_string_resource (si->dpy,
+                                        "splash.help.label",
 					"Dialog.Button.Label");
 
   if (!sp->heading_label)
@@ -212,27 +216,27 @@ make_splash_dialog (saver_info *si)
     sp->heading_label = s;
   }
 
-  f = get_string_resource ("splash.headingFont", "Dialog.Font");
+  f = get_string_resource (si->dpy, "splash.headingFont", "Dialog.Font");
   sp->heading_font = XLoadQueryFont (si->dpy, (f ? f : "fixed"));
   if (!sp->heading_font) sp->heading_font = XLoadQueryFont (si->dpy, "fixed");
   if (f) free (f);
 
-  f = get_string_resource("splash.bodyFont", "Dialog.Font");
+  f = get_string_resource(si->dpy, "splash.bodyFont", "Dialog.Font");
   sp->body_font = XLoadQueryFont (si->dpy, (f ? f : "fixed"));
   if (!sp->body_font) sp->body_font = XLoadQueryFont (si->dpy, "fixed");
   if (f) free (f);
 
-  f = get_string_resource("splash.buttonFont", "Dialog.Font");
+  f = get_string_resource(si->dpy, "splash.buttonFont", "Dialog.Font");
   sp->button_font = XLoadQueryFont (si->dpy, (f ? f : "fixed"));
   if (!sp->button_font) sp->button_font = XLoadQueryFont (si->dpy, "fixed");
   if (f) free (f);
 
-  sp->foreground = get_pixel_resource ("splash.foreground",
-				       "Dialog.Foreground",
-				       si->dpy, cmap);
-  sp->background = get_pixel_resource ("splash.background",
-				       "Dialog.Background",
-				       si->dpy, cmap);
+  sp->foreground = get_pixel_resource (si->dpy, cmap,
+                                       "splash.foreground",
+				       "Dialog.Foreground");
+  sp->background = get_pixel_resource (si->dpy, cmap, 
+                                       "splash.background",
+				       "Dialog.Background");
 
   if (sp->foreground == sp->background)
     {
@@ -241,26 +245,30 @@ make_splash_dialog (saver_info *si)
       sp->background = WhitePixelOfScreen (ssi->screen);
     }
 
-  sp->button_foreground = get_pixel_resource ("splash.Button.foreground",
-					      "Dialog.Button.Foreground",
-					      si->dpy, cmap);
-  sp->button_background = get_pixel_resource ("splash.Button.background",
-					      "Dialog.Button.Background",
-					      si->dpy, cmap);
-  sp->shadow_top = get_pixel_resource ("splash.topShadowColor",
-				       "Dialog.Foreground",
-				       si->dpy, cmap);
-  sp->shadow_bottom = get_pixel_resource ("splash.bottomShadowColor",
-					  "Dialog.Background",
-					  si->dpy, cmap);
+  sp->button_foreground = get_pixel_resource (si->dpy, cmap,
+                                              "splash.Button.foreground",
+					      "Dialog.Button.Foreground");
+  sp->button_background = get_pixel_resource (si->dpy, cmap,
+                                              "splash.Button.background",
+					      "Dialog.Button.Background");
+  sp->shadow_top = get_pixel_resource (si->dpy, cmap,
+                                       "splash.topShadowColor",
+				       "Dialog.Foreground");
+  sp->shadow_bottom = get_pixel_resource (si->dpy, cmap,
+                                          "splash.bottomShadowColor",
+					  "Dialog.Background");
 
-  sp->logo_width = get_integer_resource ("splash.logo.width",
+  sp->logo_width = get_integer_resource (si->dpy, 
+                                         "splash.logo.width",
 					 "Dialog.Logo.Width");
-  sp->logo_height = get_integer_resource ("splash.logo.height",
+  sp->logo_height = get_integer_resource (si->dpy, 
+                                          "splash.logo.height",
 					  "Dialog.Logo.Height");
-  sp->internal_border = get_integer_resource ("splash.internalBorderWidth",
+  sp->internal_border = get_integer_resource (si->dpy, 
+                                              "splash.internalBorderWidth",
 					      "Dialog.InternalBorderWidth");
-  sp->shadow_width = get_integer_resource ("splash.shadowThickness",
+  sp->shadow_width = get_integer_resource (si->dpy, 
+                                           "splash.shadowThickness",
 					   "Dialog.ShadowThickness");
 
   if (sp->logo_width == 0)  sp->logo_width = 150;
@@ -393,7 +401,9 @@ make_splash_dialog (saver_info *si)
     if (y < sy) y = sy;
   }
 
-  bw = get_integer_resource ("splash.borderWidth", "Dialog.BorderWidth");
+  bw = get_integer_resource (si->dpy, 
+                             "splash.borderWidth",
+                             "Dialog.BorderWidth");
 
   si->splash_dialog =
     XCreateWindow (si->dpy,
@@ -404,10 +414,13 @@ make_splash_dialog (saver_info *si)
 		   attrmask, &attrs);
   XSetWindowBackground (si->dpy, si->splash_dialog, sp->background);
 
-  sp->logo_pixmap = xscreensaver_logo (si->dpy, si->splash_dialog, cmap,
+  sp->logo_pixmap = xscreensaver_logo (ssi->screen, 
+                                       /* same visual as si->splash_dialog */
+                                       DefaultVisualOfScreen (ssi->screen),
+                                       si->splash_dialog, cmap,
                                        sp->background, 
                                        &sp->logo_pixels, &sp->logo_npixels,
-                                       0, True);
+                                       &sp->logo_clipmask, True);
 
   XMapRaised (si->dpy, si->splash_dialog);
   XSync (si->dpy, False);
@@ -554,6 +567,8 @@ draw_splash_window (saver_info *si)
       XGetGeometry (si->dpy, sp->logo_pixmap, &root, &x, &y, &w, &h, &bw, &d);
       XSetForeground (si->dpy, gc1, sp->foreground);
       XSetBackground (si->dpy, gc1, sp->background);
+      XSetClipMask (si->dpy, gc1, sp->logo_clipmask);
+      XSetClipOrigin (si->dpy, gc1, x1 + ((x2 - (int)w) /2), y1 + ((y2 - (int)h) / 2));
       if (d == 1)
         XCopyPlane (si->dpy, sp->logo_pixmap, si->splash_dialog, gc1,
                     0, 0, w, h,
@@ -671,6 +686,8 @@ destroy_splash_window (saver_info *si)
 
   if (sp->logo_pixmap)
     XFreePixmap (si->dpy, sp->logo_pixmap);
+  if (sp->logo_clipmask)
+    XFreePixmap (si->dpy, sp->logo_clipmask);
   if (sp->logo_pixels)
     {
       if (sp->logo_npixels)
@@ -689,6 +706,7 @@ void
 handle_splash_event (saver_info *si, XEvent *event)
 {
   splash_dialog_data *sp = si->sp_data;
+  saver_screen_info *ssi = sp->prompt_screen;
   int which = 0;
   if (!sp) return;
 
@@ -732,13 +750,14 @@ handle_splash_event (saver_info *si, XEvent *event)
 	  if (which && sp->pressed == which)
 	    {
 	      destroy_splash_window (si);
+              sp = si->sp_data;
 	      switch (which)
 		{
-		case 1: do_demo (si); break;
+		case 1: do_demo (ssi); break;
 #ifdef PREFS_BUTTON
-		case 2: do_prefs (si); break;
+		case 2: do_prefs (ssi); break;
 #endif /* PREFS_BUTTON */
-		case 3: do_help (si); break;
+		case 3: do_help (ssi); break;
 		default: abort();
 		}
 	    }
@@ -747,8 +766,9 @@ handle_splash_event (saver_info *si, XEvent *event)
               /* click and release on the window but not in a button:
                  treat that as "dismiss the splash dialog." */
 	      destroy_splash_window (si);
+              sp = si->sp_data;
             }
-	  sp->pressed = 0;
+	  if (sp) sp->pressed = 0;
 	  update_splash_window (si);
 	}
       break;
@@ -774,76 +794,49 @@ unsplash_timer (XtPointer closure, XtIntervalId *id)
 # define fork  vfork
 #endif /* VMS */
 
-static void
-fork_and_exec (saver_info *si, const char *command, const char *desc)
-{
-  saver_preferences *p = &si->prefs;
-  pid_t forked;
-  char buf [512];
-  char *av[5];
-  int ac;
-
-  if (!command || !*command)
-    {
-      fprintf (stderr, "%s: no %s command has been specified.\n",
-	       blurb(), desc);
-      return;
-    }
-
-  switch ((int) (forked = fork ()))
-    {
-    case -1:
-      sprintf (buf, "%s: couldn't fork", blurb());
-      perror (buf);
-      break;
-
-    case 0:
-      close (ConnectionNumber (si->dpy));		/* close display fd */
-      hack_subproc_environment (si->default_screen);	/* set $DISPLAY */
-      ac = 0;
-      av [ac++] = (char *) p->shell;
-      av [ac++] = (char *) "-c";
-      av [ac++] = (char *) command;
-      av [ac]   = 0;
-      execvp (av[0], av);				/* shouldn't return. */
-
-      sprintf (buf, "%s: execvp(\"%s\", \"%s\", \"%s\") failed",
-	       blurb(), av[0], av[1], av[2]);
-      perror (buf);
-      fflush (stderr);
-      fflush (stdout);
-      exit (1);			 /* Note that this only exits a child fork.  */
-      break;
-
-    default:
-      /* parent fork. */
-      break;
-    }
-}
-
 
 static void
-do_demo (saver_info *si)
+do_demo (saver_screen_info *ssi)
 {
+  saver_info *si = ssi->global;
   saver_preferences *p = &si->prefs;
-  fork_and_exec (si, p->demo_command, "demo-mode");
+  const char *cmd = p->demo_command;
+
+  if (cmd && *cmd)
+    fork_and_exec (ssi, cmd);
+  else
+    fprintf (stderr, "%s: no demo-mode command has been specified.\n",
+             blurb());
 }
 
 #ifdef PREFS_BUTTON
 static void
-do_prefs (saver_info *si)
+do_prefs (saver_screen_info *ssi)
 {
+  saver_info *si = ssi->global;
   saver_preferences *p = &si->prefs;
-  fork_and_exec (si, p->prefs_command, "preferences");
+  const char *cmd = p->prefs_command;
+
+  if (command && *command)
+    fork_and_exec (ssi, cmd);
+  else
+    fprintf (stderr, "%s: no preferences command has been specified.\n",
+             blurb());
 }
 #endif /* PREFS_BUTTON */
 
 static void
-do_help (saver_info *si)
+do_help (saver_screen_info *ssi)
 {
+  saver_info *si = ssi->global;
   saver_preferences *p = &si->prefs;
-  char *help_command;
+  char *help_command = 0;
 
+  if (!p->load_url_command || !*p->load_url_command)
+    {
+      fprintf (stderr, "%s: no URL command has been specified.\n", blurb());
+      return;
+    }
   if (!p->help_url || !*p->help_url)
     {
       fprintf (stderr, "%s: no Help URL has been specified.\n", blurb());
@@ -851,8 +844,10 @@ do_help (saver_info *si)
     }
 
   help_command = (char *) malloc (strlen (p->load_url_command) +
-				  (strlen (p->help_url) * 2) + 10);
-  sprintf (help_command, p->load_url_command, p->help_url, p->help_url);
-  fork_and_exec (si, help_command, "URL-loading");
+				  (strlen (p->help_url) * 4) + 10);
+  sprintf (help_command, p->load_url_command,
+           p->help_url, p->help_url, p->help_url, p->help_url);
+
+  fork_and_exec (ssi, help_command);
   free (help_command);
 }
