@@ -107,14 +107,20 @@
 #include "usleep.h"
 #include "yarandom.h"
 #include "resources.h"
+#include "vroot.h"
 
 
 /* Globals *******************************************************************/
 char*        progname;
-XtAppContext app;
-XrmDatabase  db;
 char*        progclass = XSUBLIM_NAME;
-char*        defaults[] =
+
+# ifdef __GNUC__
+  __extension__   /* don't warn about "string length is greater than the
+                     length ISO C89 compilers are required to support"
+                     in the following string constant... */
+# endif
+
+static char*        defaults[] =
 {
 	".background:                     #000000",
 	".foreground:                     #FFFFFF",
@@ -172,7 +178,8 @@ char*        defaults[] =
 	"*" XSUBLIM_ARG_CENTER":          true",
 	NULL
 };
-XrmOptionDescRec options[] =
+
+static XrmOptionDescRec options[] =
 {
 	{"-" XSUBLIM_ARG_FONT,          "." XSUBLIM_ARG_FONT,
 	 XrmoptionSepArg,0},
@@ -215,7 +222,7 @@ static int Xsublim_Sig_Last;
 /* Functions *****************************************************************/
 
 /* Defer signals to protect the server grab ================================ */
-void xsublim_Sig_Catch(int sig_Number)
+static void xsublim_Sig_Catch(int sig_Number)
 {
 	/* BSD needs this reset each time, and it shouldn't hurt anything
 	   else */
@@ -223,120 +230,16 @@ void xsublim_Sig_Catch(int sig_Number)
 	Xsublim_Sig_Last = sig_Number;
 }
 
-/* Get the screensaver's window ============================================ */
-static XErrorHandler Xsublim_Ss_Handler = NULL;
-static int           Xsublim_Ss_Status;
-
 /* This was all basically swiped from driver/remote.c and util/vroot.h */
-static int xsublim_Ss_Handler(Display* handle_Display,
-            XErrorEvent* handle_Error)
-{
-	if (handle_Error->error_code == BadWindow)
-	{
-		Xsublim_Ss_Status = BadWindow;
-		return 0;
-	}
-	if (Xsublim_Ss_Handler == NULL)
-	{
-		fprintf(stderr,"%x: ",progname);
-		abort();
-	}
-	return (*Xsublim_Ss_Handler)(handle_Display,handle_Error);
-}
 static Window xsublim_Ss_GetWindow(Display* ss_Display)
 {
-	Window        win_Root;
-	Window        win_RootReturn;
-	Window        win_Parent;
-	Window*       win_Child;
-	Window        win_Win;
-	int           child_Count;
-	int           child_Index;
-	Atom          prop_Type;
-	int           prop_Format;
-	unsigned long prop_Count;
-	unsigned long prop_Bytes;
-	char*         prop_Value;
-	int           prop_Status;
-	static Atom   XA_SCREENSAVER_VERSION = -1;
-	static Atom   __SWM_VROOT;
-
-	/* Assume bad things */
-	win_Win = 0;
-	win_Child = NULL;
-
-	/* Find the atoms */
-	if (XA_SCREENSAVER_VERSION == -1)
-	{
-		XA_SCREENSAVER_VERSION = XInternAtom(ss_Display,
-		 "_SCREENSAVER_VERSION",FALSE);
-		__SWM_VROOT = XInternAtom(ss_Display,"__SWM_VROOT",FALSE);
-	}
-
-	/* Find a screensaver window */
-	win_Root = RootWindowOfScreen(DefaultScreenOfDisplay(ss_Display));
-	if (XQueryTree(ss_Display,win_Root,&win_RootReturn,&win_Parent,
-	 &win_Child,&child_Count) != FALSE)
-	{
-		if (
-		 (win_Root == win_RootReturn) &&
-		 (win_Parent == 0) &&
-		 (win_Child != NULL) &&
-		 (child_Count > 0))
-		{
-			for (child_Index = 0;child_Index < child_Count;
-			 child_Index++)
-			{
-				XSync(ss_Display,FALSE);
-				Xsublim_Ss_Status = 0;
-				Xsublim_Ss_Handler =
-				 XSetErrorHandler(xsublim_Ss_Handler);
-				prop_Value = NULL;
-				prop_Status = XGetWindowProperty(ss_Display,
-				 win_Child[child_Index],XA_SCREENSAVER_VERSION,
-				 0,200,FALSE,XA_STRING,&prop_Type,&prop_Format,
-				 &prop_Count,&prop_Bytes,
-				 (unsigned char**)&prop_Value);
-				XSync(ss_Display,FALSE);
-				XSetErrorHandler(Xsublim_Ss_Handler);
-				if (prop_Value != NULL)
-				{
-					XFree(prop_Value);
-				}
-				if (Xsublim_Ss_Status == BadWindow)
-				{
-					prop_Status = BadWindow;
-				}
-				if ((prop_Status == Success) &&
-				 (prop_Type != None))
-				{
-					/* See if it's a virtual root */
-					prop_Value = NULL;
-					prop_Status =
-					 XGetWindowProperty(ss_Display,
-					 win_Child[child_Index],__SWM_VROOT,0,
-					 1,FALSE,XA_WINDOW,&prop_Type,
-					 &prop_Format,&prop_Count,&prop_Bytes,
-					 (unsigned char**)&prop_Value);
-					if (prop_Value != NULL)
-					{
-						XFree(prop_Value);
-					}
-					if ((prop_Status == Success) &&
-					 (prop_Type != None))
-					{
-						win_Win =
-						 win_Child[child_Index];
-					}
-				}
-			}
-		}
-	}
-	if (win_Child != NULL)
-	{
-		XFree(win_Child);
-	}
-	return win_Win;
+  Screen *s = DefaultScreenOfDisplay (ss_Display);
+  Window root = XRootWindowOfScreen (s);
+  Window vroot = VirtualRootWindowOfScreen (s);
+  if (root == vroot)
+    return 0;
+  else
+    return vroot;
 }
 
 /* Main ==================================================================== */
@@ -358,8 +261,11 @@ static int xsublim_Sh_Handler(Display* handle_Display,
 	}
 	return (*Xsublim_Sh_Handler)(handle_Display,handle_Error);
 }
+
+
 int main(int argc,char* argv[])
 {
+	XtAppContext app;
 	int               sig_Number;
 	int               sig_Signal[] =
 	{
@@ -394,7 +300,7 @@ int main(int argc,char* argv[])
 		-1
 	};
 	Widget            app_App;
-	Display*          disp_Display;
+	Display*          dpy;
 	Window            win_Root;
 	XWindowAttributes attr_Win;
 	XGCValues         gc_ValFore;
@@ -515,23 +421,22 @@ int main(int argc,char* argv[])
             exit (-1);
           }
 
-	disp_Display = XtDisplay(app_App);
-	db = XtDatabase(disp_Display);
-	XtGetApplicationNameAndClass(disp_Display,&progname,&progclass);
+	dpy = XtDisplay(app_App);
+	XtGetApplicationNameAndClass(dpy,&progname,&progclass);
 	win_Root = RootWindowOfScreen(XtScreen(app_App));
 	XtDestroyWidget(app_App);
 
 	/* Get the arguments */
-	arg_FlagCenter = get_boolean_resource(XSUBLIM_ARG_CENTER,"Boolean");
-	arg_FlagOutline = get_boolean_resource(XSUBLIM_ARG_OUTLINE,"Boolean");
-	arg_FlagScreensaver = get_boolean_resource(XSUBLIM_ARG_SCREENSAVER,
+	arg_FlagCenter = get_boolean_resource(dpy, XSUBLIM_ARG_CENTER,"Boolean");
+	arg_FlagOutline = get_boolean_resource(dpy, XSUBLIM_ARG_OUTLINE,"Boolean");
+	arg_FlagScreensaver = get_boolean_resource(dpy, XSUBLIM_ARG_SCREENSAVER,
 	 "Boolean");
-	arg_FlagRandom = get_boolean_resource(XSUBLIM_ARG_RANDOM,"Boolean");
-	arg_DelayShow = get_integer_resource(XSUBLIM_ARG_DELAYSHOW,"Integer");
-	arg_DelayWord = get_integer_resource(XSUBLIM_ARG_DELAYWORD,"Integer");
-	arg_DelayPhraseMin = get_integer_resource(XSUBLIM_ARG_DELAYPHRASEMIN,
+	arg_FlagRandom = get_boolean_resource(dpy, XSUBLIM_ARG_RANDOM,"Boolean");
+	arg_DelayShow = get_integer_resource(dpy, XSUBLIM_ARG_DELAYSHOW,"Integer");
+	arg_DelayWord = get_integer_resource(dpy, XSUBLIM_ARG_DELAYWORD,"Integer");
+	arg_DelayPhraseMin = get_integer_resource(dpy, XSUBLIM_ARG_DELAYPHRASEMIN,
 	 "Integer");
-	arg_DelayPhraseMax = get_integer_resource(XSUBLIM_ARG_DELAYPHRASEMAX,
+	arg_DelayPhraseMax = get_integer_resource(dpy, XSUBLIM_ARG_DELAYPHRASEMAX,
 	 "Integer");
 	if (arg_DelayPhraseMax < arg_DelayPhraseMin)
 	{
@@ -543,7 +448,7 @@ int main(int argc,char* argv[])
 	text_Item = 0;
 	text_Count = 0;
 	memset(text_Used,0,sizeof(text_Used));
-	arg_Source = get_string_resource(XSUBLIM_ARG_FILE,"Filename");
+	arg_Source = get_string_resource(dpy, XSUBLIM_ARG_FILE,"Filename");
 	if (arg_Source != NULL)
 	{
 		FILE*       file_Fs;
@@ -576,7 +481,7 @@ int main(int argc,char* argv[])
 	}
 	else
 	{
-		arg_Source = get_string_resource(XSUBLIM_ARG_PROGRAM,
+		arg_Source = get_string_resource(dpy, XSUBLIM_ARG_PROGRAM,
 		 "Executable");
 		if (arg_Source != NULL)
 		{
@@ -627,7 +532,7 @@ int main(int argc,char* argv[])
 		else
 		{
 			arg_Text =
-			 get_string_resource(XSUBLIM_ARG_PHRASES,"Phrases");
+			 get_string_resource(dpy, XSUBLIM_ARG_PHRASES,"Phrases");
 			if (arg_Text != NULL)
 			{
 				arg_Text = strdup(arg_Text);
@@ -652,12 +557,12 @@ int main(int argc,char* argv[])
 	}
 
 	/* Load the font */
-	font_Font = XLoadQueryFont(disp_Display,
-	 get_string_resource(XSUBLIM_ARG_FONT,"Font"));
+	font_Font = XLoadQueryFont(dpy,
+	 get_string_resource(dpy, XSUBLIM_ARG_FONT,"Font"));
 	font_Index = 0;
 	while ((font_Font == NULL) && (font_List[font_Index] != NULL))
 	{
-		font_Font = XLoadQueryFont(disp_Display,font_List[font_Index]);
+		font_Font = XLoadQueryFont(dpy,font_List[font_Index]);
 		font_Index++;
 	}
 	if (font_Font == NULL)
@@ -667,22 +572,26 @@ int main(int argc,char* argv[])
 	}
 
 	/* Create the GCs */
-	XGetWindowAttributes(disp_Display,win_Root,&attr_Win);
+	XGetWindowAttributes(dpy,win_Root,&attr_Win);
 	gc_ValFore.font = font_Font->fid;
-	gc_ValFore.foreground = get_pixel_resource("foreground","Foreground",
-	 disp_Display,attr_Win.colormap);
-	gc_ValFore.background = get_pixel_resource("background","Background",
-	 disp_Display,attr_Win.colormap);
+	gc_ValFore.foreground = get_pixel_resource(dpy,
+                                                   attr_Win.colormap,
+                                                   "foreground","Foreground");
+	gc_ValFore.background = get_pixel_resource(dpy,
+                                                   attr_Win.colormap,
+                                                   "background","Background");
 	gc_ValFore.subwindow_mode = IncludeInferiors;
-	gc_GcFore = XCreateGC(disp_Display,win_Root,
+	gc_GcFore = XCreateGC(dpy,win_Root,
 	 (GCFont|GCForeground|GCBackground|GCSubwindowMode),&gc_ValFore);
 	gc_ValBack.font = font_Font->fid;
-	gc_ValBack.foreground = get_pixel_resource("background","Background",
-	 disp_Display,attr_Win.colormap);
-	gc_ValBack.background = get_pixel_resource("foreground","Foreground",
-	 disp_Display,attr_Win.colormap);
+	gc_ValBack.foreground = get_pixel_resource(dpy,
+                                                   attr_Win.colormap,
+                                                   "background","Background");
+	gc_ValBack.background = get_pixel_resource(dpy,
+                                                   attr_Win.colormap,
+                                                   "foreground","Foreground");
 	gc_ValBack.subwindow_mode = IncludeInferiors;
-	gc_GcBack = XCreateGC(disp_Display,win_Root,
+	gc_GcBack = XCreateGC(dpy,win_Root,
 	 (GCFont|GCForeground|GCBackground|GCSubwindowMode),&gc_ValBack);
 
 	/* Loop ------------------------------------------------------------ */
@@ -694,7 +603,7 @@ int main(int argc,char* argv[])
 		if (arg_FlagScreensaver != FALSE)
 		{
 			/* Find the screensaver's window */
-			win_Root = xsublim_Ss_GetWindow(disp_Display);
+			win_Root = xsublim_Ss_GetWindow(dpy);
 			if (win_Root == 0)
 			{
 				usleep(30000000);
@@ -736,7 +645,7 @@ int main(int argc,char* argv[])
 			/* Once-per-word stuff ----------------------------- */
 
 			/* Find the text's position */
-			XGetWindowAttributes(disp_Display,win_Root,&attr_Win);
+			XGetWindowAttributes(dpy,win_Root,&attr_Win);
 			text_Length = strlen(text_Word);
 			text_Width = XTextWidth(font_Font,text_Word,
 			 text_Length)+XSUBLIM_TEXT_OUTLINE*2;
@@ -782,9 +691,9 @@ int main(int argc,char* argv[])
 
 			/* Grab the server -- we can't let anybody draw over
 			   us */
-			XSync(disp_Display,FALSE);
-			XGrabServer(disp_Display);
-			XSync(disp_Display,FALSE);
+			XSync(dpy,FALSE);
+			XGrabServer(dpy);
+			XSync(dpy,FALSE);
 
 			/* Set up an error handler that ignores BadMatches --
 			   since the screensaver can take its window away at
@@ -794,7 +703,7 @@ int main(int argc,char* argv[])
 			 XSetErrorHandler(xsublim_Sh_Handler);
 
 			/* Save the current background */
-			image_Image = XGetImage(disp_Display,win_Root,image_X,
+			image_Image = XGetImage(dpy,win_Root,image_X,
 			 image_Y,image_Width,image_Height,~0L,ZPixmap);
 
 			/* If we've successfully saved the background... */
@@ -817,7 +726,7 @@ int main(int argc,char* argv[])
 							   annoying at some
 							   point... */
 							XDrawString(
-							 disp_Display,
+							 dpy,
 							 win_Root,gc_GcBack,
 							 text_X+text_Outline[
 							 text_OutlineIndex].
@@ -835,7 +744,7 @@ int main(int argc,char* argv[])
 					}
 
 					/* Draw the word */
-					XDrawString(disp_Display,win_Root,
+					XDrawString(dpy,win_Root,
 					 gc_GcFore,text_X,
 					 text_Y+(font_Font->ascent),text_Word,
 					 text_Length);
@@ -843,14 +752,14 @@ int main(int argc,char* argv[])
 				if (Xsublim_Sh_Status == 0)
 				{
 					/* Wait a bit */
-					XSync(disp_Display,FALSE);
+					XSync(dpy,FALSE);
 					if (Xsublim_Sig_Last == -1)
 					{
 						usleep(arg_DelayShow);
 					}
 	
 					/* Restore the background */
-					XPutImage(disp_Display,win_Root,
+					XPutImage(dpy,win_Root,
 					 gc_GcFore,image_Image,0,0,image_X,
 					 image_Y,image_Width,image_Height);
 				}
@@ -860,10 +769,10 @@ int main(int argc,char* argv[])
 			}
 
 			/* Restore the error handler, ungrab the server */
-                        XSync(disp_Display,FALSE);
+                        XSync(dpy,FALSE);
 			XSetErrorHandler(Xsublim_Sh_Handler);
-			XUngrabServer(disp_Display);
-                        XSync(disp_Display,FALSE);
+			XUngrabServer(dpy);
+                        XSync(dpy,FALSE);
 
 			/* Pause between words */
 			if (Xsublim_Sig_Last == -1)
